==> kaffe-0.5p3.tgz: kaffe-0.5p3/kaffevm/jit/registers.h <==
/* registers.h
 * Instruction macros to extract information.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, June 1996.
 */

#ifndef __registers_h
#define __registers_h

#define	Reserved		0x00
#define	Rint			0x01
#define	Rlong			0x02
#define	Rfloat			0x04
#define	Rdouble			0x08
#define	Rref			0x10

typedef struct _kregs {
	struct _slots*		slot;
	uint8			ctype;
	uint8			type;
	uint8			readonce;
	uint32			used;
} kregs;

extern kregs reginfo[];
extern int usecnt;

#define	register_invalidate(r)	reginfo[r].slot = NOSLOT

#define	MAXREG			NR_REGISTERS
#define	NOREG			MAXREG

#define	SLOT2FRAMEOFFSET(_s)					\
	(((_s) - slotinfo < maxArgs) ? 				\
		SLOT2ARGOFFSET((_s) - slotinfo + isStatic) :	\
		SLOT2LOCALOFFSET((_s) - slotinfo))

void	spill(slots*);
void	reload(slots*);

int	slowSlotRegister(slots*, int, int);
int	fastSlotRegister(slots*, int, int);
void	clobberRegister(int);
void	forceRegister(slots*, int, int);

#define	rread			1
#define	rwrite			2

#define	_slowSlotRegister(_s, _t, _u) 					\
	(((reginfo[(_s)->regno].ctype & (_t)) != (_t)) ?		\
		slowSlotRegister(_s, _t, _u) :				\
		fastSlotRegister(_s, _t, _u))

#define	rreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rread)
#define	wreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rwrite)
#define	rwreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rread|rwrite)
#define	rslot_int(i)		slowSlotOffset(seq_slot(s, i), Rint, rread)
#define	wslot_int(i)		slowSlotOffset(seq_slot(s, i), Rint, rwrite)
#define	const_int(i)		s->u[i].iconst

#define	rreg_long(i)		_slowSlotRegister(seq_slot(s, i), Rlong, rread)
#define	wreg_long(i)		_slowSlotRegister(seq_slot(s, i), Rlong, rwrite)
#define	rslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rread)
#define	rslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rread)
#define	wslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rwrite)

#define	rreg_float(i)		_slowSlotRegister(seq_slot(s, i), Rfloat, rread)
#define	wreg_float(i)		_slowSlotRegister(seq_slot(s, i), Rfloat, rwrite)
#define	rslot_float(i)		slowSlotOffset(seq_slot(s, i), Rfloat, rread)
#define	wslot_float(i)		slowSlotOffset(seq_slot(s, i), Rfloat, rwrite)
#define	const_float(i)		s->u[i].fconst

#define	rreg_double(i)		_slowSlotRegister(seq_slot(s, i), Rdouble, rread)
#define	wreg_double(i)		_slowSlotRegister(seq_slot(s, i), Rdouble, rwrite)
#define	rslot_double(i)		slowSlotOffset(seq_slot(s, i), Rdouble, rread)
#define	wslot_double(i)		slowSlotOffset(seq_slot(s, i), Rdouble, rwrite)
#define	const_double(i)		s->u[i].fconst

#endif

==> kaffe-0.5.5.tgz: kaffe-0.5.5/kaffevm/jit/registers.h <==
/* registers.h
 * Instruction macros to extract information.
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>
 */

#ifndef __registers_h
#define __registers_h

#define	Reserved		0x00
#define	Rint			0x01
#define	Rlong			0x02
#define	Rfloat			0x04
#define	Rdouble			0x08
#define	Rref			0x10

typedef struct _kregs {
	struct _slots*		slot;
	uint8			ctype;
	uint8			type;
	uint8			readonce;
	uint32			used;
} kregs;

extern kregs reginfo[];
extern int usecnt;

#define	register_invalidate(r)	reginfo[r].slot = NOSLOT

#define	MAXREG			NR_REGISTERS
#define	NOREG			MAXREG

#define	SLOT2FRAMEOFFSET(_s)					\
	(((_s) - slotinfo < maxArgs) ? 				\
		SLOT2ARGOFFSET((_s) - slotinfo + isStatic) :	\
		SLOT2LOCALOFFSET((_s) - slotinfo))

void	spill(slots*);
void	reload(slots*);

int	slowSlotRegister(slots*, int, int);
int	fastSlotRegister(slots*, int, int);
void	clobberRegister(int);
void	forceRegister(slots*, int, int);

#define	rread			1
#define	rwrite			2

#define	_slowSlotRegister(_s, _t, _u) 					\
	(((reginfo[(_s)->regno].type & (_t)) != (_t)) ?			\
		slowSlotRegister(_s, _t, _u) :				\
		fastSlotRegister(_s, _t, _u))

#define	rreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rread)
#define	wreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rwrite)
#define	rwreg_int(i)		_slowSlotRegister(seq_slot(s, i), Rint, rread|rwrite)
#define	rslot_int(i)		slowSlotOffset(seq_slot(s, i), Rint, rread)
#define	wslot_int(i)		slowSlotOffset(seq_slot(s, i), Rint, rwrite)
#define	const_int(i)		s->u[i].iconst

#define	rreg_long(i)		_slowSlotRegister(seq_slot(s, i), Rlong, rread)
#define	wreg_long(i)		_slowSlotRegister(seq_slot(s, i), Rlong, rwrite)
#define	rslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rread)
#define	rslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rread)
#define	wslot_long(i)		slowSlotOffset(seq_slot(s, i), Rlong, rwrite)

#define	rreg_float(i)		_slowSlotRegister(seq_slot(s, i), Rfloat, rread)
#define	wreg_float(i)		_slowSlotRegister(seq_slot(s, i), Rfloat, rwrite)
#define	rslot_float(i)		slowSlotOffset(seq_slot(s, i), Rfloat, rread)
#define	wslot_float(i)		slowSlotOffset(seq_slot(s, i), Rfloat, rwrite)
#define	const_float(i)		s->u[i].fconst

#define	rreg_double(i)		_slowSlotRegister(seq_slot(s, i), Rdouble, rread)
#define	wreg_double(i)		_slowSlotRegister(seq_slot(s, i), Rdouble, rwrite)
#define	rslot_double(i)		slowSlotOffset(seq_slot(s, i), Rdouble, rread)
#define	wslot_double(i)		slowSlotOffset(seq_slot(s, i), Rdouble, rwrite)
#define	const_double(i)		s->u[i].fconst

#endif
